# -*- coding: utf-8 -*-
"""
    lantz.drivers.basler
    ~~~~~~~~~~~~~~~~~~~~~~
    :company: Basler
    :description: Manufactures cameras industrial applications
    :website: https://baslerweb.com/en
    ----
    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""


from .basler import BaslerCam

__all__ = ['BaslerCam']
