# -*- coding: utf-8 -*-
"""
    lantz.drivers.princetoninstruments.winspec
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    Implementation of Winspec PI Camera over a Socket
    Author: Berk Diler
    Date: 29/08/2017
"""


from .winspec import Winspec

__all__ = ['Winspec']
