# -*- coding: utf-8 -*-
"""
    lantz.drivers.pco
    ~~~~~~~~~~~~~~~~~

    :company: PCO.
    :description: High performance CCD-, CMOS- and sCMOS camera systems.
    :website: http://www.pco.de

    ---

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .sensicam import Sensicam

__all__ = ['Sensicam', ]
