# -*- coding: utf-8 -*-
"""
    lantz.drivers.legacy.aa
    ~~~~~~~~~~~~~~~~~~~~~~~

    :company: AA Opto Electronic.
    :description: Radio frequency and acousto-optic devices, Laser based sub-systems.
    :website: http://opto.braggcell.com/

    ----

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .aotf import MDSnC

__all__ = ['MDSnC', ]

