# -*- coding: utf-8 -*-
"""
    lantz.drivers.legacy.rgblasersystems
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :company: RGB Lasersysteme GmbH.
    :description: Lasers and Lasers Systems.
    :website: http://www.rgb-laser.com/

    ----

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .minilasevo import MiniLasEvo

__all__ = ['MiniLasEvo']
