# -*- coding: utf-8 -*-
"""
    lantz.drivers.signalrecovery
    ~~~~~~~~~~~~~~~~~~~~~

    :company: Signal Recovery
    :description: Measurement Technologies
    :website: http://www.signalrecovery.com/

    ---

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD,

"""

from .sr7265 import SR7265

__all__ = ['SR7265']
