# -*- coding: utf-8 -*-
"""
    lantz.drivers.mpb
    ~~~~~~~~~~~~~~~~~

    :company: MPB Communications Inc.
    :description: Laser products.
    :website: http://www.mpbc.ca/

    ----

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .vfl import VFL

__all__ = ['VFL']
