# -*- coding: utf-8 -*-
"""
    lantz.drivers.labjack._internal
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Python drivers provided by LabJack.

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""
