# -*- coding: utf-8 -*-
"""
    lantz.drivers.yokogawa
    ~~~~~~~~~~~~~~~~~~~~~

    :company: Yokogawa
    :description: Measurement instruments
    :website: http://www.yokogawa.com/

    ---
"""
from .yokogawa7651 import Yokogawa7651

__all__ = ['Yokogawa7651']
