# -*- coding: utf-8 -*-
"""
    lantz.drivers.legacy
    ~~~~~~~~~~~~~~~~~~~~

    Legacy drivers based on old instrument classes. Do not use for new code.

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

