# -*- coding: utf-8 -*-
"""
    lantz.drivers.prior
    ~~~~~~~~~~~~~~~~~~~

    :company: PRIOR Scientific
    :description: Microscope Automation & Custom Solutions
    :website: http://www.prior.com/

    ----

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .nanoscanz import NanoScanZ

__all__ = ['NanoScanZ']
