# -*- coding: utf-8 -*-
"""
    lantz.drivers.kentech
    ~~~~~~~~~~~~~~~~~~~~~

    :company: Kentech Instruments Ltd.
    :description: Manufacturers of specialised and custom built electronics and imaging equipment.
    :website: http://www.kentech.co.uk/

    ----

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .hri import HRI

__all__ = ['HRI', ]
