# -*- coding: utf-8 -*-
"""
    lantz.drivers.legacy.newport
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :company: Newport.
    :description: Test and Measurement Equipment.
    :website: http://www.newport.com/

    ---

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD,

"""

from .powermeter1830c import powermeter1830c

__all__ = ['powermeter1830c']
