"""
This constant file was automatically generated by a quick script I wrote for the enum part and h2py for the constant part.  It aims
to help out remembering the enum values and constants, but probably as bugs... please refer to the actual documentation for the correct values (if something is not working)
and feel free to correct the values and submit a pull request.
"""


class EWfmItem:
	WfmItem_MARKER1 = 1
	WfmItem_MARKER2 = 2
	WfmItem_DIGITAL0 = 3
	WfmItem_DIGITAL1 = 4
	WfmItem_DIGITAL2 = 5
	WfmItem_DIGITAL3 = 6
	WfmItem_DIGITAL4 = 7
	WfmItem_DIGITAL5 = 8
	WfmItem_DIGITAL6 = 9
	WfmItem_DIGITAL7 = 10
	WfmItem_DIGITAL8 = 11
	WfmItem_DIGITAL9 = 12
	WfmItem_DIGITAL10 = 13
	WfmItem_DIGITAL11 = 14
	WfmItem_DIGITAL12 = 15
	WfmItem_DIGITAL13 = 16
class ELogicSignature:
	LogicSignature_0 = 1
	LogicSignature_1 = 2
	LogicSignature_MIX = 3
class EStdWfmType:
	StdWfmType_SQUARE = 1
	StdWfmType_TRIANGLE = 2
	StdWfmType_RAMP = 3
	StdWfmType_GAUSSIAN_NOISE = 4
class EPointDrawInterpolationType:
	PointDrawInterpolationType_SPLINE = 1
class EDualWfmMathType:
	MATH_SUB = 1
class EMarkDensity:
	MARK_DENSITY_1_4 = 1
	MARK_DENSITY_1_2 = 2
	MARK_DENSITY_3_4 = 3
class ESeqItem:
	ITEM_WFM_CH2 = 1
	ITEM_WFM_CH3 = 2
	ITEM_WFM_CH4 = 3
	ITEM_WAIT = 4
	ITEM_LOOP = 5
	ITEM_JUMP_TO = 6
	ITEM_GO_TO = 7
class ECalDiagType:
	CAL_DIAG_TYPE_YIG = 1
	CAL_DIAG_TYPE_CLOCKPOWER = 2
	CAL_DIAG_TYPE_IPC = 3
	CAL_DIAG_TYPE_DTIMING = 4
	CAL_DIAG_TYPE_ANALOG = 5
	CAL_DIAG_TYPE_MARKER = 6
	CAL_DIAG_TYPE_MDELAY = 7
	CAL_DIAG_TYPE_MCPOINT = 8
	CAL_DIAG_TYPE_SKEW = 9
class EUserCalDiagType:
	USER_CALDIAG_TYPE_D_AWG_REGISTER = 1
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH1 = 2
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH2 = 3
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH3 = 4
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH4 = 5
	USER_CALDIAG_TYPE_D_CLOCK_REGISTER = 6
	USER_CALDIAG_TYPE_D_CLOCK_PLL = 7
	USER_CALDIAG_TYPE_D_OUTPUT1_REGISTER = 8
	USER_CALDIAG_TYPE_D_OUTPUT1_ANALOG = 9
	USER_CALDIAG_TYPE_D_OUTPUT1_MARKER = 10
	USER_CALDIAG_TYPE_D_OUTPUT2_REGISTER = 11
	USER_CALDIAG_TYPE_D_OUTPUT2_ANALOG = 12
	USER_CALDIAG_TYPE_D_OUTPUT2_MARKER = 13
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH1 = 14
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH1 = 15
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH2 = 16
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH2 = 17
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH3 = 18
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH3 = 19
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH4 = 20
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH4 = 21
	USER_CALDIAG_TYPE_D_AWG_DTIMING = 22
	USER_CALDIAG_TYPE_D_OUTPUT1_ANALOG_2 = 23
class EClockSource:
	ClockSource_INTERNAL = 1
	ClockSource_EXTERNAL = 2
class EReferenceSource:
	ReferenceSource_INTERNAL = 1
	ReferenceSource_EXTERNAL = 2
class EReferenceType:
	ReferenceType_Fixed = 1
	ReferenceType_Variable = 2
class EReferenceClockFrequency:
	ReferenceClockFrequency_10MHZ = 1
	ReferenceClockFrequency_20MHZ = 2
	ReferenceClockFrequency_100MHZ = 3
class EDividerRate:
	DividerRate_DIV_1 = 1
	DividerRate_DIV_2 = 2
	DividerRate_DIV_4 = 3
	DividerRate_DIV_8 = 4
	DividerRate_DIV_16 = 5
	DividerRate_DIV_32 = 6
	DividerRate_DIV_64 = 7
	DividerRate_DIV_128 = 8
	DividerRate_DIV_256 = 9
class EExternalReferenceStatus:
	ExternalReferenceStatus_INPUT_OK = 1
	ExternalReferenceStatus_INPUT_MISSING = 2
	ExternalReferenceStatus_INPUT_OUT_OF_RANGE = 3
	ExternalReferenceStatus_INPUT_UNSTABLE = 4
class EExternalOscillatorStatus:
	ExternalOscillatorStatus_INPUT_OK = 1
	ExternalOscillatorStatus_INPUT_MISSING = 2
	ExternalOscillatorStatus_INPUT_OUT_OF_RANGE = 3
	ExternalOscillatorStatus_INPUT_UNSTABLE = 4
class ETriggerSource:
	TriggerSource_EXTERNAL = 1
	TriggerSource_INTERNAL = 2
class ETriggerInputImpedance:
	TriggerInputImpedance_50OHM = 1
	TriggerInputImpedance_1KOHM = 2
class ETriggerInputSlope:
	TriggerInputSlope_POL_POSITIVE = 1
	TriggerInputSlope_POL_NEGATIVE = 2
class ETriggerInputPolarity:
	TriggerInputPolarity_POSITIVE = 1
	TriggerInputPolarity_NEGATIVE = 2
class EEventInputImpedance:
	EventInputImpedance_50OHM = 1
	EventInputImpedance_1KOHM = 2
class EEventInputPolarity:
	EventInputPolarity_POSITIVE = 1
	EventInputPolarity_NEGATIVE = 2
class EJumpTiming:
	JumpTiming_SYNC = 1
	JumpTiming_ASYNC = 2
class EAnalogInputMethod:
	AnalogInputMethod_IMAmplitudeOffset = 1
	AnalogInputMethod_IMHighLow = 2
class EDacResolution:
	DacResolution_8 = 1
	DacResolution_10 = 2
	DacResolution_14 = 3
class EFilter:
	Filter_20MHZ = 1
	Filter_50MHZ = 2
	Filter_100MHZ = 3
	Filter_200MHZ = 4
	Filter_THROUGH = 5
class EMarkerInputMethod:
	MarkerInputMethod_IMAmplitudeOffset = 1
	MarkerInputMethod_IMHighLow = 2
class EChannelCoupling:
	ChannelCoupling_Off = 1
	ChannelCoupling_Pair = 2
	ChannelCoupling_All = 3
class ERunMode:
	RunMode_CONTINUOUS = 1
	RunMode_TRIGGERED = 2
	RunMode_GATED = 3
	RunMode_SEQUENCE = 4
class ESequencerState:
	SequencerState_STOPPED = 1
	SequencerState_RUNNING = 2
	SequencerState_WAIT_FOR_TRIGGER = 3
class ESequencerType:
	SequencerType_HW = 1
	SequencerType_SW = 2
class EWaitValue:
	WaitValue_FIRST = 1
	WaitValue_LAST = 2
class EJumpTargetType:
	JumpTargetType_OFF = 1
	JumpTargetType_INDEX = 2
	JumpTargetType_NEXT = 3
class EWaveformType:
	WaveformType_INTEGER = 1
	WaveformType_REAL = 2
class EWfmDacResolution:
	WfmDacResolution_8 = 1
	WfmDacResolution_10 = 2
	WfmDacResolution_14 = 3
class EOverHeat:
	OverHeat_None = 1
	OverHeat_Message = 2
	OverHeat_Shutdown = 3
class EDigitalInputMethod:
	DigitalInputMethod_IMAmplitudeOffset = 1
	DigitalInputMethod_IMHighLow = 2
class ENormalizationType:
	NormalizationType_NONE = 1
	NormalizationType_FULL_SCALE = 2
	NormalizationType_ZERO_REFERENCE = 3
class EPhaseDelayInputMethod:
	PhaseDelayInputMethod_PHASE = 1
	PhaseDelayInputMethod_DELAY_IN_TIME = 2
class ETeklinkConnectionType:
	TeklinkConnectionType_NONE = 1
	TeklinkConnectionType_HUB = 2
	TeklinkConnectionType_INSTRUMENT = 3
class ETeklinkCpState:
	TeklinkCpState_NOTRUNNING = 1
	TeklinkCpState_RUNNING = 2
	TeklinkCpState_PENDING = 3
class EJumpMode:
	JumpMode_EVENT = 1
	JumpMode_TABLE = 2

class Commands:
	CS_MAGIC = "MAGIC"
	CS_VERSION = "VERSION"
	CS_SAMPLING_RATE = "SAMPLING_RATE"
	CS_REPETITION_RATE = "REPETITION_RATE"
	CS_HOLD_REPETITION_RATE = "HOLD_REPETITION_RATE"
	CS_CLOCK_SOURCE = "CLOCK_SOURCE"
	CS_REFERENCE_SOURCE = "REFERENCE_SOURCE"
	CS_EXTERNAL_REFERENCE_TYPE = "EXTERNAL_REFERENCE_TYPE"
	CS_REFERENCE_CLOCK_FREQUENCY_SELECTION = "REFERENCE_CLOCK_FREQUENCY_SELECTION"
	CS_REFERENCE_MULTIPLIER_RATE = "REFERENCE_MULTIPLIER_RATE"
	CS_DIVIDER_RATE = "DIVIDER_RATE"
	CS_TRIGGER_SOURCE = "TRIGGER_SOURCE"
	CS_INTERNAL_TRIGGER_RATE = "INTERNAL_TRIGGER_RATE"
	CS_TRIGGER_INPUT_IMPEDANCE = "TRIGGER_INPUT_IMPEDANCE"
	CS_TRIGGER_INPUT_SLOPE = "TRIGGER_INPUT_SLOPE"
	CS_TRIGGER_INPUT_POLARITY = "TRIGGER_INPUT_POLARITY"
	CS_TRIGGER_INPUT_THRESHOLD = "TRIGGER_INPUT_THRESHOLD"
	CS_EVENT_INPUT_IMPEDANCE = "EVENT_INPUT_IMPEDANCE"
	CS_EVENT_INPUT_POLARITY = "EVENT_INPUT_POLARITY"
	CS_EVENT_INPUT_THRESHOLD = "EVENT_INPUT_THRESHOLD"
	CS_JUMP_TIMING = "JUMP_TIMING"
	CS_INTERLEAVE = "INTERLEAVE"
	CS_ZEROING = "ZEROING"
	CS_INTERLEAVE_ADJ_PHASE = "INTERLEAVE_ADJ_PHASE"
	CS_INTERLEAVE_ADJ_AMPLITUDE = "INTERLEAVE_ADJ_AMPLITUDE"
	CS_COUPLING = "COUPLING"
	CS_RUN_MODE = "RUN_MODE"
	CS_WAIT_VALUE = "WAIT_VALUE"
	CS_RUN_STATE = "RUN_STATE"
	CS_EVENT_JUMP_MODE = "EVENT_JUMP_MODE"
	CS_TABLE_JUMP_STROBE = "TABLE_JUMP_STROBE"
	CS_TABLE_JUMP_DEFINITION = "TABLE_JUMP_DEFINITION"
	CS_DAC_RESOLUTION_1 = "DAC_RESOLUTION_1"
	CS_OUTPUT_WAVEFORM_NAME_1 = "OUTPUT_WAVEFORM_NAME_1"
	CS_CHANNEL_STATE_1 = "CHANNEL_STATE_1"
	CS_ANALOG_DIRECT_OUTPUT_1 = "ANALOG_DIRECT_OUTPUT_1"
	CS_ANALOG_FILTER_1 = "ANALOG_FILTER_1"
	CS_ANALOG_METHOD_1 = "ANALOG_METHOD_1"
	CS_ANALOG_AMPLITUDE_1 = "ANALOG_AMPLITUDE_1"
	CS_ANALOG_OFFSET_1 = "ANALOG_OFFSET_1"
	CS_ANALOG_HIGH_1 = "ANALOG_HIGH_1"
	CS_ANALOG_LOW_1 = "ANALOG_LOW_1"
	CS_MARKER1_SKEW_1 = "MARKER1_SKEW_1"
	CS_MARKER1_METHOD_1 = "MARKER1_METHOD_1"
	CS_MARKER1_AMPLITUDE_1 = "MARKER1_AMPLITUDE_1"
	CS_MARKER1_OFFSET_1 = "MARKER1_OFFSET_1"
	CS_MARKER1_HIGH_1 = "MARKER1_HIGH_1"
	CS_MARKER1_LOW_1 = "MARKER1_LOW_1"
	CS_MARKER2_SKEW_1 = "MARKER2_SKEW_1"
	CS_MARKER2_METHOD_1 = "MARKER2_METHOD_1"
	CS_MARKER2_AMPLITUDE_1 = "MARKER2_AMPLITUDE_1"
	CS_MARKER2_OFFSET_1 = "MARKER2_OFFSET_1"
	CS_MARKER2_HIGH_1 = "MARKER2_HIGH_1"
	CS_MARKER2_LOW_1 = "MARKER2_LOW_1"
	CS_DIGITAL_METHOD_1 = "DIGITAL_METHOD_1"
	CS_DIGITAL_AMPLITUDE_1 = "DIGITAL_AMPLITUDE_1"
	CS_DIGITAL_OFFSET_1 = "DIGITAL_OFFSET_1"
	CS_DIGITAL_HIGH_1 = "DIGITAL_HIGH_1"
	CS_DIGITAL_LOW_1 = "DIGITAL_LOW_1"
	CS_EXTERNAL_ADD_1 = "EXTERNAL_ADD_1"
	CS_PHASE_DELAY_INPUT_METHOD_1 = "PHASE_DELAY_INPUT_METHOD_1"
	CS_PHASE_1 = "PHASE_1"
	CS_DELAY_IN_TIME_1 = "DELAY_IN_TIME_1"
	CS_DELAY_IN_POINTS_1 = "DELAY_IN_POINTS_1"
	CS_CHANNEL_SKEW_1 = "CHANNEL_SKEW_1"
	CS_DAC_RESOLUTION_2 = "DAC_RESOLUTION_2"
	CS_OUTPUT_WAVEFORM_NAME_2 = "OUTPUT_WAVEFORM_NAME_2"
	CS_CHANNEL_STATE_2 = "CHANNEL_STATE_2"
	CS_ANALOG_DIRECT_OUTPUT_2 = "ANALOG_DIRECT_OUTPUT_2"
	CS_ANALOG_FILTER_2 = "ANALOG_FILTER_2"
	CS_ANALOG_METHOD_2 = "ANALOG_METHOD_2"
	CS_ANALOG_AMPLITUDE_2 = "ANALOG_AMPLITUDE_2"
	CS_ANALOG_OFFSET_2 = "ANALOG_OFFSET_2"
	CS_ANALOG_HIGH_2 = "ANALOG_HIGH_2"
	CS_ANALOG_LOW_2 = "ANALOG_LOW_2"
	CS_MARKER1_SKEW_2 = "MARKER1_SKEW_2"
	CS_MARKER1_METHOD_2 = "MARKER1_METHOD_2"
	CS_MARKER1_AMPLITUDE_2 = "MARKER1_AMPLITUDE_2"
	CS_MARKER1_OFFSET_2 = "MARKER1_OFFSET_2"
	CS_MARKER1_HIGH_2 = "MARKER1_HIGH_2"
	CS_MARKER1_LOW_2 = "MARKER1_LOW_2"
	CS_MARKER2_SKEW_2 = "MARKER2_SKEW_2"
	CS_MARKER2_METHOD_2 = "MARKER2_METHOD_2"
	CS_MARKER2_AMPLITUDE_2 = "MARKER2_AMPLITUDE_2"
	CS_MARKER2_OFFSET_2 = "MARKER2_OFFSET_2"
	CS_MARKER2_HIGH_2 = "MARKER2_HIGH_2"
	CS_MARKER2_LOW_2 = "MARKER2_LOW_2"
	CS_DIGITAL_METHOD_2 = "DIGITAL_METHOD_2"
	CS_DIGITAL_AMPLITUDE_2 = "DIGITAL_AMPLITUDE_2"
	CS_DIGITAL_OFFSET_2 = "DIGITAL_OFFSET_2"
	CS_DIGITAL_HIGH_2 = "DIGITAL_HIGH_2"
	CS_DIGITAL_LOW_2 = "DIGITAL_LOW_2"
	CS_EXTERNAL_ADD_2 = "EXTERNAL_ADD_2"
	CS_PHASE_DELAY_INPUT_METHOD_2 = "PHASE_DELAY_INPUT_METHOD_2"
	CS_PHASE_2 = "PHASE_2"
	CS_DELAY_IN_TIME_2 = "DELAY_IN_TIME_2"
	CS_DELAY_IN_POINTS_2 = "DELAY_IN_POINTS_2"
	CS_CHANNEL_SKEW_2 = "CHANNEL_SKEW_2"
	CS_DAC_RESOLUTION_3 = "DAC_RESOLUTION_3"
	CS_OUTPUT_WAVEFORM_NAME_3 = "OUTPUT_WAVEFORM_NAME_3"
	CS_CHANNEL_STATE_3 = "CHANNEL_STATE_3"
	CS_ANALOG_DIRECT_OUTPUT_3 = "ANALOG_DIRECT_OUTPUT_3"
	CS_ANALOG_FILTER_3 = "ANALOG_FILTER_3"
	CS_ANALOG_METHOD_3 = "ANALOG_METHOD_3"
	CS_ANALOG_AMPLITUDE_3 = "ANALOG_AMPLITUDE_3"
	CS_ANALOG_OFFSET_3 = "ANALOG_OFFSET_3"
	CS_ANALOG_HIGH_3 = "ANALOG_HIGH_3"
	CS_ANALOG_LOW_3 = "ANALOG_LOW_3"
	CS_MARKER1_SKEW_3 = "MARKER1_SKEW_3"
	CS_MARKER1_METHOD_3 = "MARKER1_METHOD_3"
	CS_MARKER1_AMPLITUDE_3 = "MARKER1_AMPLITUDE_3"
	CS_MARKER1_OFFSET_3 = "MARKER1_OFFSET_3"
	CS_MARKER1_HIGH_3 = "MARKER1_HIGH_3"
	CS_MARKER1_LOW_3 = "MARKER1_LOW_3"
	CS_MARKER2_SKEW_3 = "MARKER2_SKEW_3"
	CS_MARKER2_METHOD_3 = "MARKER2_METHOD_3"
	CS_MARKER2_AMPLITUDE_3 = "MARKER2_AMPLITUDE_3"
	CS_MARKER2_OFFSET_3 = "MARKER2_OFFSET_3"
	CS_MARKER2_HIGH_3 = "MARKER2_HIGH_3"
	CS_MARKER2_LOW_3 = "MARKER2_LOW_3"
	CS_DIGITAL_METHOD_3 = "DIGITAL_METHOD_3"
	CS_DIGITAL_AMPLITUDE_3 = "DIGITAL_AMPLITUDE_3"
	CS_DIGITAL_OFFSET_3 = "DIGITAL_OFFSET_3"
	CS_DIGITAL_HIGH_3 = "DIGITAL_HIGH_3"
	CS_DIGITAL_LOW_3 = "DIGITAL_LOW_3"
	CS_EXTERNAL_ADD_3 = "EXTERNAL_ADD_3"
	CS_PHASE_DELAY_INPUT_METHOD_3 = "PHASE_DELAY_INPUT_METHOD_3"
	CS_PHASE_3 = "PHASE_3"
	CS_DELAY_IN_TIME_3 = "DELAY_IN_TIME_3"
	CS_DELAY_IN_POINTS_3 = "DELAY_IN_POINTS_3"
	CS_CHANNEL_SKEW_3 = "CHANNEL_SKEW_3"
	CS_DAC_RESOLUTION_4 = "DAC_RESOLUTION_4"
	CS_OUTPUT_WAVEFORM_NAME_4 = "OUTPUT_WAVEFORM_NAME_4"
	CS_CHANNEL_STATE_4 = "CHANNEL_STATE_4"
	CS_ANALOG_DIRECT_OUTPUT_4 = "ANALOG_DIRECT_OUTPUT_4"
	CS_ANALOG_FILTER_4 = "ANALOG_FILTER_4"
	CS_ANALOG_METHOD_4 = "ANALOG_METHOD_4"
	CS_ANALOG_AMPLITUDE_4 = "ANALOG_AMPLITUDE_4"
	CS_ANALOG_OFFSET_4 = "ANALOG_OFFSET_4"
	CS_ANALOG_HIGH_4 = "ANALOG_HIGH_4"
	CS_ANALOG_LOW_4 = "ANALOG_LOW_4"
	CS_MARKER1_SKEW_4 = "MARKER1_SKEW_4"
	CS_MARKER1_METHOD_4 = "MARKER1_METHOD_4"
	CS_MARKER1_AMPLITUDE_4 = "MARKER1_AMPLITUDE_4"
	CS_MARKER1_OFFSET_4 = "MARKER1_OFFSET_4"
	CS_MARKER1_HIGH_4 = "MARKER1_HIGH_4"
	CS_MARKER1_LOW_4 = "MARKER1_LOW_4"
	CS_MARKER2_SKEW_4 = "MARKER2_SKEW_4"
	CS_MARKER2_METHOD_4 = "MARKER2_METHOD_4"
	CS_MARKER2_AMPLITUDE_4 = "MARKER2_AMPLITUDE_4"
	CS_MARKER2_OFFSET_4 = "MARKER2_OFFSET_4"
	CS_MARKER2_HIGH_4 = "MARKER2_HIGH_4"
	CS_MARKER2_LOW_4 = "MARKER2_LOW_4"
	CS_DIGITAL_METHOD_4 = "DIGITAL_METHOD_4"
	CS_DIGITAL_AMPLITUDE_4 = "DIGITAL_AMPLITUDE_4"
	CS_DIGITAL_OFFSET_4 = "DIGITAL_OFFSET_4"
	CS_DIGITAL_HIGH_4 = "DIGITAL_HIGH_4"
	CS_DIGITAL_LOW_4 = "DIGITAL_LOW_4"
	CS_EXTERNAL_ADD_4 = "EXTERNAL_ADD_4"
	CS_PHASE_DELAY_INPUT_METHOD_4 = "PHASE_DELAY_INPUT_METHOD_4"
	CS_PHASE_4 = "PHASE_4"
	CS_DELAY_IN_TIME_4 = "DELAY_IN_TIME_4"
	CS_DELAY_IN_POINTS_4 = "DELAY_IN_POINTS_4"
	CS_CHANNEL_SKEW_4 = "CHANNEL_SKEW_4"
	CS_DAC_RESOLUTION_N = "DAC_RESOLUTION_"
	CS_OUTPUT_WAVEFORM_NAME_N = "OUTPUT_WAVEFORM_NAME_"
	CS_CHANNEL_STATE_N = "CHANNEL_STATE_"
	CS_ANALOG_DIRECT_OUTPUT_N = "ANALOG_DIRECT_OUTPUT_"
	CS_ANALOG_FILTER_N = "ANALOG_FILTER_"
	CS_ANALOG_METHOD_N = "ANALOG_METHOD_"
	CS_ANALOG_AMPLITUDE_N = "ANALOG_AMPLITUDE_"
	CS_ANALOG_OFFSET_N = "ANALOG_OFFSET_"
	CS_ANALOG_HIGH_N = "ANALOG_HIGH_"
	CS_ANALOG_LOW_N = "ANALOG_LOW_"
	CS_MARKER1_SKEW_N = "MARKER1_SKEW_"
	CS_MARKER1_METHOD_N = "MARKER1_METHOD_"
	CS_MARKER1_AMPLITUDE_N = "MARKER1_AMPLITUDE_"
	CS_MARKER1_OFFSET_N = "MARKER1_OFFSET_"
	CS_MARKER1_HIGH_N = "MARKER1_HIGH_"
	CS_MARKER1_LOW_N = "MARKER1_LOW_"
	CS_MARKER2_SKEW_N = "MARKER2_SKEW_"
	CS_MARKER2_METHOD_N = "MARKER2_METHOD_"
	CS_MARKER2_AMPLITUDE_N = "MARKER2_AMPLITUDE_"
	CS_MARKER2_OFFSET_N = "MARKER2_OFFSET_"
	CS_MARKER2_HIGH_N = "MARKER2_HIGH_"
	CS_MARKER2_LOW_N = "MARKER2_LOW_"
	CS_DIGITAL_METHOD_N = "DIGITAL_METHOD_"
	CS_DIGITAL_AMPLITUDE_N = "DIGITAL_AMPLITUDE_"
	CS_DIGITAL_OFFSET_N = "DIGITAL_OFFSET_"
	CS_DIGITAL_HIGH_N = "DIGITAL_HIGH_"
	CS_DIGITAL_LOW_N = "DIGITAL_LOW_"
	CS_EXTERNAL_ADD_N = "EXTERNAL_ADD_"
	CS_PHASE_DELAY_INPUT_METHOD_N = "PHASE_DELAY_INPUT_METHOD_"
	CS_PHASE_N = "PHASE_"
	CS_DELAY_IN_TIME_N = "DELAY_IN_TIME_"
	CS_DELAY_IN_POINTS_N = "DELAY_IN_POINTS_"
	CS_CHANNEL_SKEW_N = "CHANNEL_SKEW_"
	CS_DC_OUTPUT_LEVEL_1 = "DC_OUTPUT_LEVEL_1"
	CS_DC_OUTPUT_LEVEL_2 = "DC_OUTPUT_LEVEL_2"
	CS_DC_OUTPUT_LEVEL_3 = "DC_OUTPUT_LEVEL_3"
	CS_DC_OUTPUT_LEVEL_4 = "DC_OUTPUT_LEVEL_4"
	CS_DC_OUTPUT_LEVEL_N = "DC_OUTPUT_LEVEL_"
	CS_WAVEFORM_NAME_N = "WAVEFORM_NAME_"
	CS_WAVEFORM_TYPE_N = "WAVEFORM_TYPE_"
	CS_WAVEFORM_LENGTH_N = "WAVEFORM_LENGTH_"
	CS_WAVEFORM_TIMESTAMP_N = "WAVEFORM_TIMESTAMP_"
	CS_WAVEFORM_DATA_N = "WAVEFORM_DATA_"
	CS_SEQUENCE_WAIT_N = "SEQUENCE_WAIT_"
	CS_SEQUENCE_LOOP_N = "SEQUENCE_LOOP_"
	CS_SEQUENCE_JUMP_N = "SEQUENCE_JUMP_"
	CS_SEQUENCE_GOTO_N = "SEQUENCE_GOTO_"
	CS_SEQUENCE_WAVEFORM_NAME_CH_N_N = "SEQUENCE_WAVEFORM_NAME_CH_"
	CS_SEQUENCE_IS_SUBSEQUENCE_N = "SEQUENCE_IS_SUBSEQ_"
	CS_SEQUENCE_SUBSEQUENCE_NAME_N = "SEQUENCE_SUBSEQ_NAME_"
	CS_SUBSEQUENCE_NAME_O = "SUBSEQ_NAME_"
	CS_SUBSEQUENCE_TIMESTAMP_O = "SUBSEQ_TIMESTAMP_"
	CS_SUBSEQUENCE_LENGTH_O = "SUBSEQ_LENGTH_"
	CS_SUBSEQUENCE_LOOP_E_O_U = "SUBSEQ_LOOP_"
	CS_SUBSEQUENCE_WAVEFORM_NAME_CH_X_E_O_U = "SUBSEQ_WAVEFORM_NAME_CH_"
SEQUENCE_JUMP_TARGET_TYPE_OFF = 0
SEQUENCE_JUMP_TARGET_TYPE_NEXT = -1
